using System;
using System.Collections;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

using iclect.ImageManager;

namespace iclect.Forms
{
	/// <summary>
	/// Summary description for Film.
	/// </summary>
	public class Film : System.Windows.Forms.UserControl
	{
		private FilmPanel panel0;
		private FilmPanel panel1;
		private System.Windows.Forms.ToolTip toolTip;
		private System.Windows.Forms.VScrollBar vScrollBar;
		private System.Windows.Forms.HScrollBar hScrollBar;
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.ScrollBar _scrollBar;
		private FilmPanels _panels;

		public Film()
		{
			SetStyle(ControlStyles.UserPaint, true);
			SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			SetStyle(ControlStyles.DoubleBuffer, true);

			_panels			= new FilmPanels(this);
			_imageFolder	= null;
			HScroll			= false;
			VScroll			= false;

			SuspendLayout();
			try
			{
				// This call is required by the Windows.Forms Form Designer.
				InitializeComponent();

				_panels.SetPanels(panel0, panel1);

				Orientation		= Orientation.Vertical;
			}
			finally
			{
				ResumeLayout(false);
			}
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this.panel0 = new iclect.Forms.FilmPanel();
			this.panel1 = new iclect.Forms.FilmPanel();
			this.toolTip = new System.Windows.Forms.ToolTip(this.components);
			this.vScrollBar = new System.Windows.Forms.VScrollBar();
			this.hScrollBar = new System.Windows.Forms.HScrollBar();
			this.SuspendLayout();
			// 
			// panel0
			// 
			this.panel0.ForeColor = System.Drawing.Color.White;
			this.panel0.ID = 0;
			this.panel0.Name = "panel0";
			this.panel0.Size = new System.Drawing.Size(96, 96);
			this.panel0.TabIndex = 0;
			// 
			// panel1
			// 
			this.panel1.ForeColor = System.Drawing.Color.White;
			this.panel1.ID = 0;
			this.panel1.Location = new System.Drawing.Point(0, 96);
			this.panel1.Name = "panel1";
			this.panel1.Size = new System.Drawing.Size(96, 96);
			this.panel1.TabIndex = 1;
			this.panel1.Visible = false;
			// 
			// toolTip
			// 
			this.toolTip.AutoPopDelay = 5000;
			this.toolTip.InitialDelay = 500;
			this.toolTip.ReshowDelay = 100;
			// 
			// vScrollBar
			// 
			this.vScrollBar.Dock = System.Windows.Forms.DockStyle.Right;
			this.vScrollBar.Location = new System.Drawing.Point(96, 0);
			this.vScrollBar.Name = "vScrollBar";
			this.vScrollBar.Size = new System.Drawing.Size(16, 208);
			this.vScrollBar.TabIndex = 3;
			this.vScrollBar.Visible = false;
			this.vScrollBar.ValueChanged += new System.EventHandler(this.OnScrollValueChanged);
			// 
			// hScrollBar
			// 
			this.hScrollBar.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.hScrollBar.Location = new System.Drawing.Point(0, 192);
			this.hScrollBar.Name = "hScrollBar";
			this.hScrollBar.Size = new System.Drawing.Size(96, 16);
			this.hScrollBar.TabIndex = 2;
			this.hScrollBar.Visible = false;
			this.hScrollBar.ValueChanged += new System.EventHandler(this.OnScrollValueChanged);
			// 
			// Film
			// 
			this.BackColor = System.Drawing.SystemColors.AppWorkspace;
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.hScrollBar,
																		  this.vScrollBar,
																		  this.panel1,
																		  this.panel0});
			this.ForeColor = System.Drawing.Color.White;
			this.Name = "Film";
			this.Size = new System.Drawing.Size(112, 208);
			this.Enter += new System.EventHandler(this.OnEnter);
			this.Leave += new System.EventHandler(this.OnLeave);
			this.Layout += new System.Windows.Forms.LayoutEventHandler(this.OnLayout);
			this.ResumeLayout(false);

		}
		#endregion

		#region Image properties
		ImageFolder _imageFolder;
		public ImageFolder ImageFolder
		{
			get { return _imageFolder; }
			set
			{
				_focusedThumbIndex = -1;
				_imageFolder = value;
				GoToFirstThumb();
				Invalidate(true);
				PerformLayout();
			}
		}

		public int ImageCount
		{
			get
			{
				if ((_imageFolder != null) && (_imageFolder.ImageFiles != null))
					return _imageFolder.ImageFiles.Length;
				else
					return 0;
			}
		}
		#endregion

		#region Focus management
		int _focusedThumbIndex = -1;
		public int FocusedThumbIndex
		{
			get { return _focusedThumbIndex; }
			set	{ FocusThumb(value, false); }
		}
		public void FocusThumb(int index, bool center)
		{
			if (_focusedThumbIndex == index)
			{
				if (_focusedThumbIndex >= 0)
				{
					if (center)
						_panels.CenterThumb(_focusedThumbIndex);
					else
						_panels.ShowThumb(_focusedThumbIndex);
				}
			}
			else
			{
				if ((index >= ImageCount) || (index < -1))
					throw new ArgumentOutOfRangeException("index");

				if (_focusedThumbIndex >= 0)
				{
					_panels.Invalidate(_focusedThumbIndex);
					_focusedThumbIndex = -1;
				}

				_focusedThumbIndex = index;

				if (_focusedThumbIndex >= 0)
				{
					_panels.Invalidate(_focusedThumbIndex);
					if (center)
						_panels.CenterThumb(_focusedThumbIndex);
					else
						_panels.ShowThumb(_focusedThumbIndex);
				}

				if (_focusedThumbIndex == -1)
					FireThumbChanged(_focusedThumbIndex, null);
				else
					FireThumbChanged(_focusedThumbIndex, _imageFolder.ImageFiles[_focusedThumbIndex]);
			}
		}
		public void CenterFocusedThumb()
		{
			if (_focusedThumbIndex < 0)
				return;
			_panels.CenterThumb(_focusedThumbIndex);
		}

		public class ThumbChangedEventArgs : EventArgs
		{
			public ThumbChangedEventArgs(int index, ImageFile imageFile)
			{
				_index		= index;
				_imageFile	= imageFile;
			}

			ImageFile _imageFile;
			public ImageFile ImageFile
			{
				get { return _imageFile; }
			}
			int _index;
			public int Index
			{
				get { return _index; }
			}
		}

		public delegate void ThumbChangedHandler(object sender, ThumbChangedEventArgs e);
		public event ThumbChangedHandler ThumbChanged;

		protected void FireThumbChanged(int index, ImageFile imageFile)
		{
			if (ThumbChanged != null)
			{
				ThumbChangedEventArgs e = new ThumbChangedEventArgs(index, imageFile);
				ThumbChanged(this, e);
			}
		}
		#endregion

		internal void SetToolTip(Control control, int index)
		{
			toolTip.Active		= false;
			ImageFile imageFile	= _imageFolder.ImageFiles[index];
			DateTime time;
			string creation;
			if (imageFile.RecordingTime == DateTime.MinValue)
			{
				time		= imageFile.CreationTime;
				creation	= "Created";
			}
			else
			{
				time		= imageFile.RecordingTime;
				creation	= "Taken";
			}
			string text			= string.Format("Image: {0}\r\n{1}: {2}\r\nSize: {3} x {4}",
				imageFile.Name,
				creation, time,
				imageFile.Width,
				imageFile.Height
				);
			toolTip.SetToolTip(control, text);
			toolTip.Active		= true;
		}

		#region Appearance
		Color _selectionColor = Color.Blue;
		[Category("Appearance")]
		public Color SelectionColor
		{
			get { return _selectionColor; }
			set { _selectionColor = value; }
		}
		Color _focusColor = Color.Blue;
		[Category("Appearance")]
		public Color FocusColor
		{
			get { return _focusColor; }
			set { _focusColor = value; }
		}
		Color _selectedFocusColor = Color.Blue;
		[Category("Appearance")]
		public Color SelectedFocusColor
		{
			get { return _selectedFocusColor; }
			set { _selectedFocusColor = value; }
		}
		[Category("Appearance")]
		public Color FilmColor
		{
			get { return this.panel0.BackColor; }
			set
			{
				panel0.BackColor = value;
				panel1.BackColor = value;
			}
		}
		[Category("Appearance")]
		public Orientation Orientation
		{
			get { return _panels.Orientation; }
			set
			{
				if (Orientation != value)
				{
					int scrollValue		= ScrollValue;
					_panels.Orientation	= value;
					ScrollValue			= scrollValue;
				}
			}
		}
		#endregion

		[Category("Layout")]
		public int ThumbSize
		{
			get { return _panels.ThumbSize; }
			set	{ _panels.ThumbSize = value; }
		}

		[Category("Behavior")]
		public int ScrollValue
		{
			get
			{
				if (Orientation == Orientation.Vertical)
					return vScrollBar.Value;
				else
					return hScrollBar.Value;
			}
			set
			{
				if (Orientation == Orientation.Vertical)
					vScrollBar.Value = value;
				else
					hScrollBar.Value = value;
			}
		}

		private void AdjustScrollBar()
		{
			int view;
			if (Orientation == Orientation.Horizontal)
			{
				vScrollBar.Visible	= false;
				_scrollBar			= hScrollBar;
				view				= Width;
			}
			else
			{
				hScrollBar.Visible	= false;
				_scrollBar			= vScrollBar;
				view				= Height;
			}

			int length		= _panels.FilmHeight;
			int thumbSkip	= _panels.ThumbSkip;
			if (length > view)
			{
				int viewRounded			= (view / thumbSkip) * thumbSkip;
				_scrollBar.Minimum		= 0;
				_scrollBar.Maximum		= length - 1;
				_scrollBar.LargeChange	= Math.Max(viewRounded - thumbSkip, thumbSkip);
				_scrollBar.SmallChange	= thumbSkip;
				_scrollBar.Visible		= true;
			}
			else
				_scrollBar.Visible		= false;
		}
		private void OnScrollValueChanged(object sender, System.EventArgs e)
		{
			_panels.ScrollValue = _scrollBar.Value;
		}

		private void OnLayout(object sender, System.Windows.Forms.LayoutEventArgs e)
		{
			AdjustScrollBar();

			int width = _panels.FilmWidth;
			if (_scrollBar.Visible)
				width += _scrollBar.Width;

			if (Orientation == Orientation.Vertical)
				Width	= width;
			else
				Height	= width;

			_panels.Layout();
		}

		private void DrawFocusCues()
		{
			_panels.InvalidateFocusCues();
		}
		private void OnEnter(object sender, System.EventArgs e)
		{
			Focus();
			DrawFocusCues();
		}
		private void OnLeave(object sender, System.EventArgs e)
		{
			DrawFocusCues();
		}
		public void SelectAll(bool imagesOnly)
		{
			if (_imageFolder == null)
				return;

			foreach (ImageFile imageFile in _imageFolder.ImageFiles)
			{
				if ((!imagesOnly) || (imageFile.LoadException == null))
					imageFile.Selected = true;
			}
			Invalidate(true);
		}
		public void ReverseSelection(bool imagesOnly)
		{
			if (_imageFolder == null)
				return;

			foreach (ImageFile imageFile in _imageFolder.ImageFiles)
			{
				if ((!imagesOnly) || (imageFile.LoadException == null))
					imageFile.Selected = !imageFile.Selected;
			}
			Invalidate(true);
		}
		public void UnselectAll()
		{
			if (_imageFolder == null)
				return;

			foreach (ImageFile imageFile in _imageFolder.ImageFiles)
				imageFile.Selected = false;
			Invalidate(true);
		}
		public void ToggleSelection()
		{
			if (_focusedThumbIndex == -1)
				return;
			ImageFile focused = _imageFolder.ImageFiles[_focusedThumbIndex];
			focused.Selected = !focused.Selected;
			_panels.Invalidate(_focusedThumbIndex);
		}
		private bool GoTo(int index, bool center)
		{
			if (index < 0)
				index = 0;
			else
			{
				int max = ImageCount - 1;
				if (index > max)
					index = max;
			}

			FocusThumb(index, center);
			return true;
		}
		public bool GoToFirstThumb()
		{
			return GoTo(0, true);
		}
		public bool GoToLastThumb()
		{
			return GoTo(ImageCount - 1, true);
		}
		public bool GoToPreviousThumb()
		{
			return GoTo(FocusedThumbIndex - 1, true);
		}
		public bool GoToNextThumb()
		{
			return GoTo(FocusedThumbIndex + 1, true);
		}
		public bool GoPageUp()
		{
			int visible = _panels.FirstVisibleIndex;
			if (visible == _focusedThumbIndex)
				visible -= Math.Max(_panels.ViewCount - 1, 1);
			return GoTo(visible, false);
		}
		public bool GoPageDown()
		{
			int visible = _panels.LastVisibleIndex;
			if (visible == _focusedThumbIndex)
				visible += Math.Max(_panels.ViewCount - 1, 1);
			return GoTo(visible, false);
		}
		protected override bool ProcessDialogKey(Keys keyData)
		{
			switch (keyData & Keys.Modifiers)
			{
				case Keys.None:
				{
					switch (keyData & Keys.KeyCode)
					{
						case Keys.Home:
							return GoToFirstThumb();
						case Keys.End:
							return GoToLastThumb();
						case Keys.Up:
							if (Orientation == Orientation.Horizontal)
								break;
							return GoToPreviousThumb();
						case Keys.Left:
							if (Orientation == Orientation.Vertical)
								break;
							return GoToPreviousThumb();
						case Keys.Down:
							if (Orientation == Orientation.Horizontal)
								break;
							return GoToNextThumb();
						case Keys.Right:
							if (Orientation == Orientation.Vertical)
								break;
							return GoToNextThumb();
						case Keys.Enter:
							ToggleSelection();
							return true;
						case Keys.Space:
							ToggleSelection();
							return GoToNextThumb();
						case Keys.PageDown:
							return GoPageDown();
						case Keys.PageUp:
							return GoPageUp();
					}
					break;
				}
			}
			return base.ProcessDialogKey(keyData);
		}
	}
}
