using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Data;
using System.Windows.Forms;

using iclect.ImageManager;

namespace iclect.Forms
{
	/// <summary>
	/// Summary description for ImagePreview.
	/// </summary>
	public class ImagePreview : System.Windows.Forms.UserControl
	{
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public ImagePreview()
		{
			SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			SetStyle(ControlStyles.ResizeRedraw, true);

			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// ImagePreview
			// 
			this.Name = "ImagePreview";
			this.Paint += new System.Windows.Forms.PaintEventHandler(this.OnPaint);

		}
		#endregion

		ImageReference _imageReference = null;
		public ImageReference ImageReference
		{
			get { return _imageReference; }
			set
			{
				if (_imageReference != value)
				{
					_imageReference = value;
					Invalidate();
				}
			}
		}

		bool _center = true;
		public bool Center
		{
			get {return _center; }
			set
			{
				if (_center != value)
				{
					_center = value;
					if (_imageReference != null)
						Invalidate();
				}
			}
		}
		int _margin = 4;
		private void OnPaint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			Image image = null;
			if (_imageReference != null)
				image = _imageReference.GetImage();

			Graphics g		= e.Graphics;
			if (image == null)
			{
				using (Brush bbrush	= new SolidBrush(BackColor))
				{
					g.FillRectangle(bbrush, e.ClipRectangle);
					return;
				}
			}

			int imageWidth	= image.Width;
			int imageHeight	= image.Height;
			int paintWidth	= Width - 2 * _margin;
			int paintHeight	= Height - 2 * _margin;

			int width;
			int height;
			if (imageWidth * paintHeight > paintWidth * imageHeight)
			{
				width	= paintWidth;
				height	= imageHeight * paintWidth / imageWidth;
			}
			else
			{
				width	= imageWidth * paintHeight / imageHeight;
				height	= paintHeight;
			}

			int x1;
			int y1;
			if (_center)
			{
				x1 = (Width - width) / 2;
				y1 = (Height - height) / 2;
			}
			else
			{
				x1 = _margin;
				y1 = _margin;
			}
			int x2 = x1 + width;
			int y2 = y1 + height;

			// Draw the margins
			//  1 +----+
			//    ++--++
			//  3 ||  || 4
			//    ++--++
			//  2 +----+

			using (Brush bbrush	= new SolidBrush(BackColor))
			{
				g.FillRectangle(bbrush, 0, 0, Width, y1);
				g.FillRectangle(bbrush, 0, y2, Width, Height - y2);
				g.FillRectangle(bbrush, 0, y1, x1, height);
				g.FillRectangle(bbrush, x2, y1, Width - x2, height);
			}

			// Draw the image
			g.InterpolationMode	= InterpolationMode.Default;
			g.DrawImage(image, x1, y1, width, height);
		}

		protected override void OnPaintBackground(System.Windows.Forms.PaintEventArgs e)
		{
		}
	}
}
