using System;
using System.ComponentModel;
using System.Runtime.InteropServices;
using System.Security;
using System.Security.Permissions;
using System.Text;
using System.Windows.Forms;

namespace iclect.Forms.Utility
{
	/// <summary>
	/// Summary description for FolderBrowserDialog.
	/// </summary>
	public sealed class FolderBrowserDialog : CommonDialog
	{
		[Guid("00000002-0000-0000-C000-000000000046"), InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
		internal interface IMalloc
		{
			IntPtr Alloc(int cb);
			IntPtr Realloc(IntPtr pv, int cb);
			void Free(IntPtr pv);
			int GetSize(IntPtr pv);
			int DidAlloc(IntPtr pv);
			void HeapMinimize();
		}
		[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Auto)]
		internal class BROWSEINFO
		{
			public IntPtr hwndOwner;
			public IntPtr pidlRoot;
			public string pszDisplayName;
			public string lpszTitle;
			public int ulFlags;
			public BrowseCallbackProc lpfn;
			public IntPtr lParam;
			public int iImage;
		}
		public delegate int BrowseCallbackProc(IntPtr hwnd, int msg, IntPtr lParam, IntPtr lpData);

		internal class User32
		{
			[DllImport("User32")]
			public static extern IntPtr SendMessage(HandleRef hWnd, int msg, int wParam, int lParam);
			[DllImport("User32", CharSet=CharSet.Auto)]
			public static extern IntPtr SendMessage(HandleRef hWnd, int msg, int wParam, string lParam);
		}	
		internal class Shell32
		{
			[DllImport("Shell32", CharSet=CharSet.Auto)]
			public static extern IntPtr SHBrowseForFolder(BROWSEINFO lpbi);
			[DllImport("Shell32")]
			public static extern int SHGetMalloc(out IMalloc ppMalloc);
			[DllImport("Shell32", CharSet=CharSet.Auto)]
			public static extern bool SHGetPathFromIDList(IntPtr pidl, StringBuilder pszPath);
			[DllImport("Shell32")]
			public static extern int SHGetSpecialFolderLocation(IntPtr hwnd, int csidl, ref IntPtr ppidl);
 		}

		public FolderBrowserDialog()
		{
			Reset();
		}
		private int FolderBrowserDialog_BrowseCallbackProc(IntPtr hwnd, int msg, IntPtr lParam, IntPtr lpData)
		{
			switch (msg)
			{
				case 1:	// BFFM_INITIALIZED
				{
					if (this.selectedPath == string.Empty)
						break;

					User32.SendMessage(new HandleRef(null, hwnd), 1127, 1, this.selectedPath);
					break;
				}
				case 2:	// BFFM_SELCHANGED
				{
					if (lParam != IntPtr.Zero)
					{
						bool gotPath	= Shell32.SHGetPathFromIDList(lParam, new StringBuilder(260));
						User32.SendMessage(new HandleRef(null, hwnd), 1125, 0, (gotPath ? 1 : 0));
					}
					break;
				}
			}
			return 0; 
		}
		private static IMalloc GetSHMalloc()
		{
			IMalloc malloc;
			Shell32.SHGetMalloc(out malloc);
			return malloc; 
		}
		public override void Reset()
		{
			this.rootFolder = 0;
			this.descriptionText = string.Empty;
			this.selectedPath = string.Empty;
			this.selectedPathNeedsCheck = false;
			this.showNewFolderButton = true;
		}
		protected override bool RunDialog(IntPtr hWndOwner)
		{
			IntPtr pidlRoot = IntPtr.Zero;
			Shell32.SHGetSpecialFolderLocation(hWndOwner, (int)this.rootFolder, ref pidlRoot);
			if (pidlRoot == IntPtr.Zero)
			{
				Shell32.SHGetSpecialFolderLocation(hWndOwner, 0, ref pidlRoot);
				if (pidlRoot == IntPtr.Zero)
					throw new Exception("FolderBrowserDialogNoRootFolder");
			}

			int flags = 64 + 1;
			if (!this.showNewFolderButton)
				flags = (flags + 512);

			Application.OleRequired();
			IntPtr pidlSelected = IntPtr.Zero;
			bool success = false;
			try
			{

				BROWSEINFO bi			= new BROWSEINFO();
				bi.pidlRoot				= pidlRoot;
				bi.hwndOwner			= hWndOwner;
				bi.pszDisplayName		= new string(char.MinValue, 260);
				bi.lpszTitle			= this.descriptionText;
				bi.ulFlags				= flags;
				bi.lpfn					= new BrowseCallbackProc(this.FolderBrowserDialog_BrowseCallbackProc);
				bi.lParam = IntPtr.Zero;
				bi.iImage = 0;
				pidlSelected = Shell32.SHBrowseForFolder(bi);
				if (pidlSelected != IntPtr.Zero)
				{
					StringBuilder szPath = new StringBuilder(260);
					if (Shell32.SHGetPathFromIDList(pidlSelected, szPath))
					{
						this.selectedPathNeedsCheck = true;
						this.selectedPath = szPath.ToString();
						success = true;
					}
				}
			}
			finally
			{
				IMalloc malloc = GetSHMalloc();
				malloc.Free(pidlRoot);
				if (pidlSelected != IntPtr.Zero)
					malloc.Free(pidlSelected);
			}
			return success; 
		}
		public string Description 
		{
			get
			{
				return this.descriptionText; 
			}
			set
			{
				this.descriptionText = ((value == null) ? string.Empty : value);
			}
		}
		public Environment.SpecialFolder RootFolder
		{
			get
			{
				return this.rootFolder; 
			}
			set
			{
				if (!Enum.IsDefined(typeof(Environment.SpecialFolder), value))
					throw new InvalidEnumArgumentException("value", (int)value, typeof(Environment.SpecialFolder));

				this.rootFolder = value;
			}
		}
		public string SelectedPath
		{
			get
			{
				if ((this.selectedPath == null) || (this.selectedPath.Length == 0))
					return this.selectedPath; 

				if (this.selectedPathNeedsCheck)
					new FileIOPermission(FileIOPermissionAccess.PathDiscovery, this.selectedPath).Demand();

				return this.selectedPath; 
			}
			set
			{
				this.selectedPath = ((value == null) ? string.Empty : value);
				this.selectedPathNeedsCheck = false;
			}
		}
		public bool ShowNewFolderButton
		{
			get
			{
				return this.showNewFolderButton; 
			}
			set
			{
				this.showNewFolderButton = value;
			} 
		}
		private string descriptionText;
		private Environment.SpecialFolder rootFolder;
		private string selectedPath;
		private bool selectedPathNeedsCheck;
		private bool showNewFolderButton;
	}
}
