using System;
using System.Drawing;

namespace iclect.ImageManager
{
	/// <summary>
	/// Summary description for ImageReference.
	/// </summary>
	public class ImageReference : IDisposable
	{
		internal ImageReference(ImageFile imageFile)
		{
			if (imageFile == null)
				throw new ArgumentNullException("imageFile");

			_imageFile	= imageFile;
			_imageFile.AddRef();
		}
		~ImageReference()
		{
			Dispose(false);
		}
		public void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}
		protected void Dispose(bool disposing)
		{
			if (_imageFile != null)
			{
				_imageFile.Release();
				_imageFile = null;
			}
		}
		public Image GetImage()
		{
			if (_imageFile == null)
				throw new ObjectDisposedException("ImageReference");
			return _imageFile.GetHighImage();
		}
		public bool IsLoaded
		{
			get
			{
				if (_imageFile == null)
					throw new ObjectDisposedException("ImageReference");
				return (_imageFile._high != null);
			}
		}
		ImageFile	_imageFile;
	}
}
