using System;
using System.Threading;

namespace iclect.ImageManager
{
	/// <summary>
	/// Summary description for ThumbLoader.
	/// </summary>
	internal class ThumbLoader
	{
		public ThumbLoader(ImageFolder folder)
		{
			_folder = folder;
		}
		public void Start()
		{
			Stop();
			_stopThumbLoader.Reset();

			_thumbLoader				= new Thread(new ThreadStart(this.LoadThumbs));
			_thumbLoader.Name			= "ThumbLoader";
			_thumbLoader.IsBackground	= true;
			_thumbLoader.Start();
		}
		public void Stop()
		{
			if (_thumbLoader == null)
				return;
			_stopThumbLoader.Set();
			_thumbLoader.Join(new TimeSpan(0, 0, 15));
			_thumbLoader = null;
		}
		private void LoadThumbs()
		{
			foreach (ImageFile imageFile in _folder.ImageFiles)
			{
				if (_stopThumbLoader.WaitOne(0, false))
				{
					Console.WriteLine("Aborting 'ThumbLoader' due to stop request!");
					break;
				}	
				imageFile.GetThumbImage();
			}
			_folder.SaveThumbsFile();
		}

		ImageFolder			_folder;
		Thread				_thumbLoader;
		ManualResetEvent	_stopThumbLoader = new ManualResetEvent(false);
	}
}
